use crate::CvfsResult;
use multi_key_manager::{file::File, BorrowedNonce, LiveKeyMaterial, OpeningNonce};
use ring::aead::{Aad, NonceSequence, OpeningKey, SealingKey};
use std::io::{Read, Seek, Write};

/// This copies the buffer during encryption so the original buffer stays untouched
#[must_use]
pub(crate) fn encrypt_file_buffered<N: NonceSequence>(
    real_file: &mut File,
    buffer: &[u8],
    //keys: &mut dyn LiveKeyMaterial,
    //key: &mut SealingKey<BorrowedNonce>,
    key: &mut SealingKey<N>,
) -> CvfsResult<()> {
    let mut buf = Vec::with_capacity(4096);
    buf.extend(buffer);

    //let mut key = keys.get_sealing_master_key();
    let aad = Aad::empty();
    key.seal_in_place_append_tag(aad, &mut buf)?;

    real_file.rewind()?;
    real_file.set_len(buf.len() as u64)?;
    real_file.write_all(&buf)?;
    real_file.sync_all()?;

    Ok(())
}

#[must_use]
pub(crate) fn decrypt_file_buffered(
    real_file: &mut File,
    //keys: &mut dyn LiveKeyMaterial,
    key: &mut OpeningKey<OpeningNonce>,
) -> CvfsResult<Vec<u8>> {
    // 4KB is a good step size for disk reads
    let mut buf = Vec::with_capacity(4096);
    real_file.rewind()?;
    real_file.read_to_end(&mut buf)?;

    //let mut key = keys.get_opening_master_key();
    let aad = Aad::empty();
    println!(" > decrypt_file_buffered: {:?}", buf.len());
    let decrypted = key.open_in_place(aad, &mut buf)?;
    let decrypted_len = decrypted.len();

    buf.truncate(decrypted_len);

    Ok(buf)
}
