pub type CvfsResult<O> = Result<O, CvfsErrors>;

#[derive(Debug, PartialEq)]
pub enum CvfsErrors {
    KeyManagementError(multi_key_manager::errors::KeyManagementError),

    /// An error occured inside ring, but this was the specific action
    LeafDecryptionError,

    /// Std::io::Error
    GeneralIO(String),
    /// A constant time error occured in the `ring` crate
    GeneralRingEncryptionError,
    MutexUnlockError(String),
}

impl From<std::io::Error> for CvfsErrors {
    fn from(e: std::io::Error) -> Self {
        CvfsErrors::GeneralIO(format!("{e:?}"))
    }
}

impl From<multi_key_manager::errors::KeyManagementError> for CvfsErrors {
    fn from(e: multi_key_manager::errors::KeyManagementError) -> Self {
        CvfsErrors::KeyManagementError(e)
    }
}

impl From<ring::error::Unspecified> for CvfsErrors {
    fn from(_: ring::error::Unspecified) -> Self {
        CvfsErrors::GeneralRingEncryptionError
    }
}
impl<T> From<std::sync::PoisonError<T>> for CvfsErrors {
    fn from(e: std::sync::PoisonError<T>) -> Self {
        CvfsErrors::MutexUnlockError(format!("{e:?}"))
    }
}
